"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.claimCertificates = exports.transferCertificates = exports.issueCertificates = void 0;
const ethers_1 = require("ethers");
const Certificate_1 = require("./Certificate");
const CertificateUtils_1 = require("./CertificateUtils");
async function issueCertificates(certificateInfo, blockchainProperties) {
    const { issuer, registry, activeUser } = blockchainProperties;
    const issuerWithSigner = issuer.connect(activeUser);
    const data = certificateInfo.map((info) => CertificateUtils_1.encodeData({
        generationStartTime: info.generationStartTime,
        generationEndTime: info.generationEndTime,
        deviceId: info.deviceId,
        metadata: info.metadata
    }));
    const batchIssueTx = await issuerWithSigner.issueBatch(certificateInfo.map((info) => info.to), certificateInfo.map((info) => info.amount), data);
    const { events } = await batchIssueTx.wait();
    let issuanceEvent;
    for (const event of events) {
        try {
            issuanceEvent = issuer.interface.parseLog(event);
        }
        catch (e) {
            issuanceEvent = registry.interface.parseLog(event);
        }
        if (issuanceEvent.name === 'IssuanceBatch') {
            break;
        }
    }
    return issuanceEvent.args[2].map((id) => id.toNumber());
}
exports.issueCertificates = issueCertificates;
async function transferCertificates(certificateBatch, blockchainProperties) {
    const certificatesPromises = certificateBatch.map((cert) => new Certificate_1.Certificate(cert.id, blockchainProperties).sync());
    const { registry, activeUser } = blockchainProperties;
    const registryWithSigner = registry.connect(activeUser);
    const activeUserAddress = await activeUser.getAddress();
    const certificates = await Promise.all(certificatesPromises);
    const transferTx = await registryWithSigner.safeBatchTransferFromMultiple(certificateBatch.map((cert) => cert.from ?? activeUserAddress), certificateBatch.map((cert) => cert.to), certificateBatch.map((cert) => cert.id), certificateBatch.map((cert) => cert.amount ??
        ethers_1.BigNumber.from(certificates.find((certificate) => certificate.id === cert.id).owners[cert.from ?? activeUserAddress] ?? 0)), certificateBatch.map(() => ethers_1.utils.randomBytes(32)) // TO-DO: replace with proper data
    );
    return transferTx;
}
exports.transferCertificates = transferCertificates;
async function claimCertificates(certificateBatch, blockchainProperties) {
    const certificatesPromises = certificateBatch.map((cert) => new Certificate_1.Certificate(cert.id, blockchainProperties).sync());
    const certificates = await Promise.all(certificatesPromises);
    const { activeUser, registry } = blockchainProperties;
    const activeUserAddress = await activeUser.getAddress();
    const registryWithSigner = registry.connect(activeUser);
    const claimTx = await registryWithSigner.safeBatchTransferAndClaimFromMultiple(certificateBatch.map((cert) => cert.from ?? activeUserAddress), certificateBatch.map((cert) => cert.to ?? cert.from ?? activeUserAddress), certificateBatch.map((cert) => cert.id), certificateBatch.map((cert) => cert.amount ??
        ethers_1.BigNumber.from(certificates.find((certificate) => certificate.id === cert.id).owners[cert.from ?? activeUserAddress] ?? 0)), certificateBatch.map(() => ethers_1.utils.randomBytes(32)), // TO-DO: replace with proper data
    certificateBatch.map((cert) => CertificateUtils_1.encodeClaimData(cert.claimData)));
    return claimTx;
}
exports.claimCertificates = claimCertificates;
//# sourceMappingURL=CertificateBatchOperations.js.map