"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.approveOperator = exports.calculateClaims = exports.calculateOwnership = exports.getAllCertificateEvents = exports.decodeEvent = exports.getAllOwnedCertificates = exports.getAllCertificates = exports.decodeData = exports.encodeData = exports.decodeClaimData = exports.encodeClaimData = void 0;
const ethers_1 = require("ethers");
const Certificate_1 = require("./Certificate");
const events_1 = require("../utils/events");
const encodeClaimData = (claimData) => {
    const { beneficiary, location, countryCode, periodStartDate, periodEndDate, purpose } = claimData;
    return ethers_1.utils.defaultAbiCoder.encode(['string', 'string', 'string', 'string', 'string', 'string'], [beneficiary, location, countryCode, periodStartDate, periodEndDate, purpose]);
};
exports.encodeClaimData = encodeClaimData;
const decodeClaimData = (encodedClaimData) => {
    const [beneficiary, location, countryCode, periodStartDate, periodEndDate, purpose] = ethers_1.utils.defaultAbiCoder.decode(['string', 'string', 'string', 'string', 'string', 'string'], encodedClaimData);
    return {
        beneficiary,
        location,
        countryCode,
        periodStartDate,
        periodEndDate,
        purpose
    };
};
exports.decodeClaimData = decodeClaimData;
const encodeData = (data) => {
    const { generationStartTime, generationEndTime, deviceId, metadata } = data;
    return ethers_1.utils.defaultAbiCoder.encode(['uint256', 'uint256', 'string', 'string'], [generationStartTime, generationEndTime, deviceId, metadata ?? '']);
};
exports.encodeData = encodeData;
const decodeData = (encodedData) => {
    const [generationStartTime, generationEndTime, deviceId, metadata] = ethers_1.utils.defaultAbiCoder.decode(['uint256', 'uint256', 'string', 'string'], encodedData);
    return {
        generationStartTime: generationStartTime.toNumber(),
        generationEndTime: generationEndTime.toNumber(),
        deviceId,
        metadata
    };
};
exports.decodeData = decodeData;
async function getAllCertificates(blockchainProperties) {
    const { registry } = blockchainProperties;
    const issuanceEvents = await events_1.getEventsFromContract(registry, registry.filters.IssuanceSingle(null, null, null));
    const certificatePromises = issuanceEvents.map((event) => new Certificate_1.Certificate(event._id.toNumber(), blockchainProperties).sync());
    return Promise.all(certificatePromises);
}
exports.getAllCertificates = getAllCertificates;
async function getAllOwnedCertificates(blockchainProperties) {
    const { registry, activeUser } = blockchainProperties;
    const owner = await activeUser.getAddress();
    const transfers = await events_1.getEventsFromContract(registry, registry.filters.TransferSingle(null, null, owner, null, null));
    const certificateIds = [
        ...new Set(transfers.map((transfer) => transfer.id.toNumber()))
    ];
    const balances = await registry.balanceOfBatch(Array(certificateIds.length).fill(owner), certificateIds);
    const available = certificateIds.filter((id, index) => !balances[index].isZero());
    const certificatePromises = available.map((id) => new Certificate_1.Certificate(id, blockchainProperties).sync());
    return Promise.all(certificatePromises);
}
exports.getAllOwnedCertificates = getAllOwnedCertificates;
async function decodeEvent(eventName, event, contract) {
    const eventBlock = await contract.provider.getBlock(event.blockHash);
    return {
        name: eventName,
        transactionHash: event.transactionHash,
        blockHash: event.blockHash,
        timestamp: eventBlock.timestamp,
        ...contract.interface.decodeEventLog(eventName, event.data, event.topics)
    };
}
exports.decodeEvent = decodeEvent;
const getAllCertificateEvents = async (certId, blockchainProperties) => {
    const { registry } = blockchainProperties;
    const getEvent = async (filter, eventName) => {
        const logs = await registry.provider.getLogs({
            ...filter,
            fromBlock: 0,
            toBlock: 'latest'
        });
        const parsedLogs = await Promise.all(logs.map(async (event) => decodeEvent(eventName, event, registry)));
        return parsedLogs.filter((event) => event._id?.toNumber() === certId || event.id?.toNumber() === certId);
    };
    const issuanceSingleEvents = await getEvent(registry.filters.IssuanceSingle(null, null, null), 'IssuanceSingle');
    const issuanceBatchEvents = await getEvent(registry.filters.IssuanceBatch(null, null, null), 'IssuanceBatch');
    const transferSingleEvents = await getEvent(registry.filters.TransferSingle(null, null, null, null, null), 'TransferSingle');
    const transferBatchEvents = await getEvent(registry.filters.TransferBatch(null, null, null, null, null), 'TransferBatch');
    const transferBatchMultipleEvents = await getEvent(registry.filters.TransferBatchMultiple(null, null, null, null, null), 'TransferBatchMultiple');
    const claimSingleEvents = await getEvent(registry.filters.ClaimSingle(null, null, null, null, null, null), 'ClaimSingle');
    const claimBatchEvents = await getEvent(registry.filters.ClaimBatch(null, null, null, null, null, null), 'ClaimBatch');
    const claimBatchMultipleEvents = await getEvent(registry.filters.ClaimBatchMultiple(null, null, null, null, null, null), 'ClaimBatchMultiple');
    return [
        ...issuanceSingleEvents,
        ...issuanceBatchEvents,
        ...transferSingleEvents,
        ...transferBatchEvents,
        ...transferBatchMultipleEvents,
        ...claimSingleEvents,
        ...claimBatchEvents,
        ...claimBatchMultipleEvents
    ];
};
exports.getAllCertificateEvents = getAllCertificateEvents;
const calculateOwnership = async (certificateId, blockchainProperties) => {
    const ownedShares = {};
    const { registry } = blockchainProperties;
    const transferSingleEvents = (await events_1.getEventsFromContract(registry, registry.filters.TransferSingle(null, null, null, null, null))).filter((event) => event.id.eq(certificateId));
    const transferBatchEvents = (await events_1.getEventsFromContract(registry, registry.filters.TransferBatch(null, null, null, null, null))).filter((e) => e.ids.some((id) => id.eq(certificateId)));
    const transferBatchMultipleEvents = (await events_1.getEventsFromContract(registry, registry.filters.TransferBatchMultiple(null, null, null, null, null))).filter((e) => e.ids.some((id) => id.eq(certificateId)));
    const allHistoricOwners = [
        ...new Set([
            ...[...transferSingleEvents, ...transferBatchEvents].map((event) => event.to),
            ...transferBatchMultipleEvents
                .map((event) => event.to)
                .reduce((a, b) => a.concat(b), [])
        ])
    ].filter((address) => address !== ethers_1.constants.AddressZero);
    const allHistoricOwnersBalances = await registry.balanceOfBatch(allHistoricOwners, Array(allHistoricOwners.length).fill(certificateId.toString()));
    allHistoricOwners.forEach((owner, index) => {
        ownedShares[owner] = allHistoricOwnersBalances[index].toString();
    });
    return ownedShares;
};
exports.calculateOwnership = calculateOwnership;
const calculateClaims = async (certificateId, blockchainProperties) => {
    const claimedShares = {};
    const { registry } = blockchainProperties;
    const claimSingleEvents = (await events_1.getEventsFromContract(registry, registry.filters.ClaimSingle(null, null, null, null, null, null))).filter((event) => event._id.eq(certificateId));
    const claimBatchEvents = (await events_1.getEventsFromContract(registry, registry.filters.ClaimBatch(null, null, null, null, null, null))).filter((e) => e._ids.some((id) => id.eq(certificateId)));
    const claimBatchMultipleEvents = (await events_1.getEventsFromContract(registry, registry.filters.ClaimBatchMultiple(null, null, null, null, null, null))).filter((e) => e._ids.some((id) => id.eq(certificateId)));
    const allHistoricClaimers = [
        ...new Set([
            ...[...claimSingleEvents, ...claimBatchEvents].map((event) => event._claimSubject),
            ...claimBatchMultipleEvents
                .map((event) => event._claimSubject)
                .reduce((a, b) => a.concat(b), [])
        ])
    ].filter((address) => address !== ethers_1.constants.AddressZero);
    const allHistoricClaimersBalances = await registry.claimedBalanceOfBatch(allHistoricClaimers, Array(allHistoricClaimers.length).fill(certificateId.toString()));
    allHistoricClaimers.forEach((owner, index) => {
        claimedShares[owner] = allHistoricClaimersBalances[index].toString();
    });
    return claimedShares;
};
exports.calculateClaims = calculateClaims;
async function approveOperator(operator, blockchainProperties) {
    const { activeUser, registry } = blockchainProperties;
    const registryWithSigner = registry.connect(activeUser);
    return registryWithSigner.setApprovalForAll(operator, true);
}
exports.approveOperator = approveOperator;
//# sourceMappingURL=CertificateUtils.js.map